package javax.net.ssl;

import java.util.HashMap;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.security.AlgorithmConstraints;

public class SSLParameters
{
    private String[] cipherSuites;
    private String[] protocols;
    private boolean wantClientAuth;
    private boolean needClientAuth;
    private String identificationAlgorithm;
    private AlgorithmConstraints algorithmConstraints;
    private Map<Integer, SNIServerName> sniNames;
    private Map<Integer, SNIMatcher> sniMatchers;
    private boolean preferLocalCipherSuites;
    private String[] applicationProtocols;
    
    public SSLParameters() {
        this.sniNames = null;
        this.sniMatchers = null;
        this.applicationProtocols = new String[0];
    }
    
    public SSLParameters(String[] cipherSuites) {
        this.sniNames = null;
        this.sniMatchers = null;
        this.applicationProtocols = new String[0];
        this.setCipherSuites(cipherSuites);
    }
    
    public SSLParameters(String[] cipherSuites, String[] protocols) {
        this.sniNames = null;
        this.sniMatchers = null;
        this.applicationProtocols = new String[0];
        this.setCipherSuites(cipherSuites);
        this.setProtocols(protocols);
    }
    
    private static String[] clone(String[] array) {
        return (array == null) ? null : ((String[])array.clone());
    }
    
    public String[] getCipherSuites() {
        return clone(this.cipherSuites);
    }
    
    public void setCipherSuites(String[] array) {
        this.cipherSuites = clone(array);
    }
    
    public String[] getProtocols() {
        return clone(this.protocols);
    }
    
    public void setProtocols(String[] array) {
        this.protocols = clone(array);
    }
    
    public boolean getWantClientAuth() {
        return this.wantClientAuth;
    }
    
    public void setWantClientAuth(boolean wantClientAuth) {
        this.wantClientAuth = wantClientAuth;
        this.needClientAuth = false;
    }
    
    public boolean getNeedClientAuth() {
        return this.needClientAuth;
    }
    
    public void setNeedClientAuth(boolean needClientAuth) {
        this.wantClientAuth = false;
        this.needClientAuth = needClientAuth;
    }
    
    public AlgorithmConstraints getAlgorithmConstraints() {
        return this.algorithmConstraints;
    }
    
    public void setAlgorithmConstraints(AlgorithmConstraints algorithmConstraints) {
        this.algorithmConstraints = algorithmConstraints;
    }
    
    public String getEndpointIdentificationAlgorithm() {
        return this.identificationAlgorithm;
    }
    
    public void setEndpointIdentificationAlgorithm(String identificationAlgorithm) {
        this.identificationAlgorithm = identificationAlgorithm;
    }
    
    public final void setServerNames(List<SNIServerName> list) {
        if (list != null) {
            if (!list.isEmpty()) {
                this.sniNames = new LinkedHashMap(list.size());
                for (SNIServerName sniServerName : list) {
                    if (this.sniNames.put(Integer.valueOf(sniServerName.getType()), sniServerName) != null) {
                        throw new IllegalArgumentException("Duplicated server name of type " + sniServerName.getType());
                    }
                    continue;
                }
            }
            else {
                this.sniNames = Collections.emptyMap();
            }
        }
        else {
            this.sniNames = null;
        }
    }
    
    public final List<SNIServerName> getServerNames() {
        if (this.sniNames != null) {
            if (!this.sniNames.isEmpty()) {
                return Collections.unmodifiableList(new ArrayList(this.sniNames.values()));
            }
            return Collections.emptyList();
        }
        else {
            return null;
        }
    }
    
    public final void setSNIMatchers(Collection<SNIMatcher> collection) {
        if (collection != null) {
            if (!collection.isEmpty()) {
                this.sniMatchers = new HashMap(collection.size());
                for (SNIMatcher sniMatcher : collection) {
                    if (this.sniMatchers.put(Integer.valueOf(sniMatcher.getType()), sniMatcher) != null) {
                        throw new IllegalArgumentException("Duplicated server name of type " + sniMatcher.getType());
                    }
                    continue;
                }
            }
            else {
                this.sniMatchers = Collections.emptyMap();
            }
        }
        else {
            this.sniMatchers = null;
        }
    }
    
    public final Collection<SNIMatcher> getSNIMatchers() {
        if (this.sniMatchers != null) {
            if (!this.sniMatchers.isEmpty()) {
                return Collections.unmodifiableList(new ArrayList(this.sniMatchers.values()));
            }
            return Collections.emptyList();
        }
        else {
            return null;
        }
    }
    
    public final void setUseCipherSuitesOrder(boolean preferLocalCipherSuites) {
        this.preferLocalCipherSuites = preferLocalCipherSuites;
    }
    
    public final boolean getUseCipherSuitesOrder() {
        return this.preferLocalCipherSuites;
    }
    
    public String[] getApplicationProtocols() {
        return (String[])this.applicationProtocols.clone();
    }
    
    public void setApplicationProtocols(String[] array) {
        if (array == null) {
            throw new IllegalArgumentException("protocols was null");
        }
        String[] applicationProtocols = (String[])array.clone();
        int length = applicationProtocols.length;
        int i = 0;
        while (i < length) {
            String s = applicationProtocols[i];
            if (s == null || s.equals("")) {
                throw new IllegalArgumentException("An element of protocols was null/empty");
            }
            ++i;
        }
        this.applicationProtocols = applicationProtocols;
        return;
    }
}
